/*  Copyright (C) MOXA Inc. All rights reserved.

    This software is distributed under the terms of the
    MOXA License.  See the file COPYING-MOXA for details.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifdef WIN32
#include <windows.h>
#else
#include <unistd.h>
#include <signal.h>
#endif
#include "mbapi.h"

typedef struct _RdReq {
	char func_code;
	unsigned short addr;
	unsigned short count;
} RdReq;

#define NUM_WRITE_VAL	10

typedef struct _WrReqs {
	unsigned short addr;
	unsigned short val[NUM_WRITE_VAL];
} WrReqs;

static RdReq rd_req[] = {
	{ 0x03, 0, 10 },
	{ 0x03, 16, 1 },	/* CJC offset value */
	{ 0x04, 0, 7 },
};

static WrReqs wr_reqs = { 0, {1,1, 1,1, 1,1,0, 1,1,0 }};	/* AI current */

static int num_rd_req = sizeof(rd_req) / sizeof(rd_req[0]);

static int quit = 0;

#ifdef WIN32
BOOL WINAPI
ConsoleEventHandler(DWORD dwCtrlType)
{
	BOOL ret;

	switch (dwCtrlType)
	{
	case CTRL_C_EVENT:
		printf("CTRL_C not allowed!\n");
		return TRUE;
	case CTRL_CLOSE_EVENT:
		printf("CLOSE_WINDOW not allowed!\n");
		return TRUE;
	case CTRL_BREAK_EVENT:
		printf("CTRL_BREAK got\n");
		ret = TRUE;
		break;
	case CTRL_LOGOFF_EVENT:
		printf("LOGOFF got\n");
		ret = FALSE;
		break;
	case CTRL_SHUTDOWN_EVENT:
		printf("SHUTDOWN got\n");
		ret = FALSE;
		break;
	}
	quit = 1;

	return ret;
}
#else
static void
sighandler(int sig)
{
	quit = 1;
}

static int
set_signals()
{
	struct sigaction sa;

	memset (&sa, 0, sizeof (sa));
	sa.sa_handler = sighandler;
	if (sigaction(SIGHUP, &sa, NULL) < 0) return -1;
	if (sigaction(SIGTERM, &sa, NULL) < 0) return -2;
	if (sigaction(SIGINT, &sa, NULL) < 0) return -3;

	return 0;
}
#endif


int
main(int argc, char *argv[])
{
	int rc;
	void *hndl;
	char *ip;
	int i, j;
	unsigned short reg_val[24];
	RdReq *p;

	if (argc > 1) ip = argv[1];
	else ip = "127.0.0.1";

#ifdef WIN32
	/* set console control handler */
	if (! SetConsoleCtrlHandler(ConsoleEventHandler, TRUE)) {
		printf("SetConsoleCtrlHandler() fail\n");
	}
#else
	if (set_signals() < 0) {
#ifdef DEBUG
		printf("set_signals() error\n");
#endif
	}
#endif

	rc = mbtcp_init();
	if (rc < 0) {
		printf("mbtcp_init() error %d\n", rc);
		return -1;
	}

	rc = mbtcp_connect(ip, 502, &hndl);
	if (rc < 0) {
		printf("mbtcp_connect() error %d\n", rc);
		return -2;
	}

	printf("==== Set write parameters ====\n");
	rc = mbtcp_write_regs(hndl, 5, wr_reqs.addr, NUM_WRITE_VAL, wr_reqs.val);
	if (rc != 0) quit = 1;
	else printf("OK\n");

	while (!quit) {
			for (i = 0, p = rd_req; i < num_rd_req && !quit; i++, p++) {
				printf("----- Get read data %d -----\n", i);
				rc = mbtcp_read_regs(hndl, 5, p->func_code, p->addr, p->count, reg_val);
				if (rc < 0) {
					printf("rc=%d\n", rc);
					quit = 1;
					break;
				} else if (rc > 0) {
					printf("rc=%d\n", rc);
					continue;
				}
				printf("Function Code: %x\n", p->func_code);
				for (j = 0; j < p->count; j++)
					printf("%04X: %u\n", p->addr+j, reg_val[j]);
				if (p->func_code == 0x04 && rc == 0) {
					for (j = 0; j < p->count; j++) {
						if (p->addr+j == 0 || p->addr+j == 2) {
							float f;
							unsigned char buf[4];
							memcpy(buf, &reg_val[j], 2);
							memcpy(&buf[2], &reg_val[j+1], 2);
							memcpy(&f, buf, 4);
							printf("AI value: %f\n", f);
						} else if (p->addr+j == 4 || p->addr+j == 5 || p->addr+j == 6) {
							short t;
							unsigned char buffer[2];
							memcpy(buffer, &reg_val[j], 2);
							memcpy(&t, buffer, 2);
							if (p->addr+j == 6)
								printf("CJC value: %d\n", t);
							else
								printf("TC value: %d\n", t);
						}
					}
				}
#ifdef WIN32
				Sleep(5*1000);
#else
				sleep(5);
#endif
			}
	}
	printf("quit\n");

	rc = mbtcp_disconnect(hndl);
	if (rc < 0) {
		printf("mbtcp_disconnect() error %d\n", rc);
		return -3;
	}

	rc = mbtcp_release();
	if (rc < 0) {
		printf("mbtcp_release() error %d\n", rc);
		return -4;
	}

	return 0;
}
