/*  Copyright (C) MOXA Inc. All rights reserved.

    This software is distributed under the terms of the
    MOXA License.  See the file COPYING-MOXA for details.
*/

#ifndef _MBAPI_H
#define _MBAPI_H

#ifdef __cplusplus
extern "C" {
#endif

// The following ifdef block is the standard way of creating macros which make exporting 
// from a DLL simpler. All files within this DLL are compiled with the MBTCP_EXPORTS
// symbol defined on the command line. this symbol should not be defined on any project
// that uses this DLL. This way any other project whose source files include this file see 
// MB_API functions as being imported from a DLL, whereas this DLL sees symbols
// defined with this macro as being exported.
#ifdef MBTCP_EXPORTS
#define MB_API __declspec(dllexport)
#else
#define MB_API __declspec(dllimport)
#endif

/*  Initialize MODBUS TCP connection environment
    Input: none
    Output: none
    Return: 0 for success and negative value for failure
*/
MB_API int mbtcp_init();

/*  Release MODBUS TCP connection environment
    Input: none
    Output: none
    Return: 0 for success and negative value for failure
*/
MB_API int mbtcp_release();

/*  Create MODBUS TCP connection handler and connect to MODBUS TCP server
    Input:
      <ip> IP address of MODBUS TCP server
      <port> port number of MODBUS TCP server
    Output:
      <hndl> MODBUS TCP connection handler
    Return: 0 for success and negative value for failure
      -1 argument error
      -2 memory allocation error
      -3 failed to connect to MODBUS TCP server
*/
MB_API int
mbtcp_connect(
char *ip,
unsigned short port,
void **hndl);

/*  Close MODBUS TCP connection and destroy connection handler
    Input:
      <hndl> MODBUS TCP connection handler
    Output: none
    Return: 0 for success and negative value for failure
      -1 argument error
*/
MB_API int
mbtcp_disconnect(void *hndl);

/*  Read register values via function code 3 or 4
    Input:
      <hndl> MODBUS TCP connection handler
      <tmout> communication timeout in second
      <reg_type> register type for function code 3 or 4
                 3: read holding register
                 4: read input register
      <start_addr> starting address
      <count> quantity of registers
    Output:
      <reg_val> an array to hold register values, where the array size must be equal to or greater than the requested quantity of registers.
    Return: 0 for success, negative value for failure from API, and positive value for failure from MODBUS TCP server
      -1 argument error
      -2 failed to send MODBUS TCP request to MODBUS TCP server
      -3 failed to receive MODBUS TCP response from MODBUS TCP server
      -4 disconnection from MODBUS TCP server
      -5 illegal MODBUS TCP response message received
      -6 unexpected MODBUS TCP response message received
      1 bad function code
      2 address out of range
      3 bad read count
      4 internal gateway failure
      6 unable to accept the MODBUS request
*/
MB_API int
mbtcp_read_regs(
void *hndl,
unsigned int tmout,
unsigned char reg_type,
unsigned short start_addr,
unsigned short count,
unsigned short reg_val[]
);

/*  Write register values via function code 6 or 16
    Input:
      <hndl> MODBUS TCP connection handler
      <tmout> communication timeout in second
      <start_addr> starting address
      <count> quantity of registers
      <reg_val> an array to hold register values
    Output: none
    Return: 0 for success, negative value for failure from API, and positive value for failure from MODBUS TCP server
      -1 argument error
      -2 failed to send MODBUS TCP request to MODBUS TCP server
      -3 failed to receive MODBUS TCP response from MODBUS TCP server
      -4 disconnection from MODBUS TCP server
      -5 illegal MODBUS TCP response message received
      -6 unexpected MODBUS TCP response message received
      1 bad function code
      2 address out of range
      3 bad write count or bad write value
      4 internal gateway failure
      6 unable to accept the MODBUS request
*/
MB_API int
mbtcp_write_regs(
void *hndl,
unsigned int tmout,
unsigned short start_addr,
unsigned short count,
unsigned short reg_val[]
);

#ifdef __cplusplus
extern "C" {
#endif

#endif
